unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, JvComponentBase, JvHidControllerClass, HID, HidUsage, StdCtrls,
  ExtCtrls, ComCtrls, JvExComCtrls, JvComCtrls, uSendIRcode;

type
  TIRData = packed record
    Protocol : Byte;
    Address : Word;
    Command : Word;
    Flags : Byte;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportIRCode = packed record
    ReportID: Byte;
    Data: TIRData;
  end;

  // kein HID-Report kann groesser als 64 Byte sein
  THidIrmpVersion = packed record
    ReportID: Byte;
    Data: Array[0..10] of AnsiChar;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportIRPollingTime = packed record
    ReportID: Byte;
    Data: Word;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportPowerOn = packed record
    ReportID: Byte;
    Data: Boolean;
  end;

type
  THidReportMinRepeats = packed record
    ReportID: Byte;
    Data: Byte;
  end;

type
  TMainForm = class(TForm)
    JvHidDeviceController: TJvHidDeviceController;
    Memo1: TMemo;
    Panel1: TPanel;
    btSetIRCode: TButton;
    btPowerOnEnable: TButton;
    btPowerOnDisable: TButton;
    btReadTrainedIRCode: TButton;
    btPowerOnEnabled: TButton;
    btReadIrmpVersion: TButton;
    TrackBar_IR_Polling: TJvTrackBar;
    Label4: TLabel;
    Label2: TLabel;
    Label1: TLabel;
    Label3: TLabel;
    TrackBar_MinRepeats: TJvTrackBar;
    btSendIRCode: TButton;
    procedure OnArrival(HidDev: TJvHidDevice);
    procedure OnRemoval(HidDev: TJvHidDevice);
    procedure OnDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
      const Data: Pointer; Size: Word);
    procedure btSetIRCodeClick(Sender: TObject);
    procedure btPowerOnEnableClick(Sender: TObject);
    procedure btPowerOnDisableClick(Sender: TObject);
    procedure btReadTrainedIRCodeClick(Sender: TObject);
    procedure btPowerOnEnabledClick(Sender: TObject);
    procedure btReadIrmpVersionClick(Sender: TObject);
    procedure OnChange_IR_Pooling(Sender: TObject);
    procedure OnChanged_IR_Pooling(Sender: TObject);
    procedure OnChange_MinRepeats(Sender: TObject);
    procedure OnChanged_MinRepeats(Sender: TObject);
    procedure btSendIRCodeClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

Type
  TIrmp = record
    Protocol : Integer;
    Name : String;
  end;

const
  VendorID = $16C0;
  ProductID = $05DF;
  ProductName = 'USB IR Remote Receiver';



//irmp const:
const
  IRMP_PROTOCOLS : Array[0..29] of TIrmp =
  (
    (Protocol : 1; Name : 'IRMP_SIRCS_PROTOCOL'),
    (Protocol : 2; Name : 'IRMP_NEC_PROTOCOL'),
    (Protocol : 3; Name : 'IRMP_SAMSUNG_PROTOCOL'),
    (Protocol : 4; Name : 'IRMP_MATSUSHITA_PROTOCOL'),
    (Protocol : 5; Name : 'IRMP_KASEIKYO_PROTOCOL'),
    (Protocol : 6; Name : 'IRMP_RECS80_PROTOCOL'),
    (Protocol : 7; Name : 'IRMP_RC5_PROTOCOL'),
    (Protocol : 8; Name : 'IRMP_DENON_PROTOCOL'),
    (Protocol : 9; Name : 'IRMP_RC6_PROTOCOL'),
    (Protocol : 10; Name : 'IRMP_SAMSUNG32_PROTOCOL'),
    (Protocol : 11; Name : 'IRMP_APPLE_PROTOCOL'),
    (Protocol : 12; Name : 'IRMP_RECS80EXT_PROTOCOL'),
    (Protocol : 13; Name : 'IRMP_NUBERT_PROTOCOL'),
    (Protocol : 14; Name : 'IRMP_BANG_OLUFSEN_PROTOCOL'),
    (Protocol : 15; Name : 'IRMP_GRUNDIG_PROTOCOL'),
    (Protocol : 16; Name : 'IRMP_NOKIA_PROTOCOL'),
    (Protocol : 17; Name : 'IRMP_SIEMENS_PROTOCOL'),
    (Protocol : 18; Name : 'IRMP_FDC_PROTOCOL'),
    (Protocol : 19; Name : 'IRMP_RCCAR_PROTOCOL'),
    (Protocol : 20; Name : 'IRMP_JVC_PROTOCOL'),
    (Protocol : 21; Name : 'IRMP_RC6A_PROTOCOL'),
    (Protocol : 22; Name : 'IRMP_NIKON_PROTOCOL'),
    (Protocol : 23; Name : 'IRMP_RUWIDO_PROTOCOL'),
    (Protocol : 24; Name : 'IRMP_IR60_PROTOCOL'),
    (Protocol : 25; Name : 'IRMP_KATHREIN_PROTOCOL'),
    (Protocol : 26; Name : 'IRMP_NETBOX_PROTOCOL'),
    (Protocol : 27; Name : 'IRMP_NEC16_PROTOCOL'),
    (Protocol : 28; Name : 'IRMP_NEC42_PROTOCOL'),
    (Protocol : 29; Name : 'IRMP_LEGO_PROTOCOL'),
    (Protocol : 30; Name : 'IRMP_THOMSON_PROTOCOL')
  );

const
  NewIRCodeAvailable = 1;
  ReadPowerOnEnabled = 2;
  ReadTrainendIRCode = 3;
  SetPowerOnEnabled = 4;
  SetTrainedIRCode = 5;
  SetIRPollingTime = 6;
  ReadIrmpVersion = 7;
  SetMinRepeats = 8;
  ResetForBootloader = 9;
  SndIRCodeOut = 10;


var
  MainForm: TMainForm;
  HIDDevice: TJvHidDevice;
  ReceivedData : TIRData;

  IRReport: THidReportIRCode;
  PowerOnReport : THidReportPowerOn;
  ReportIRPollingTime : THidReportIRPollingTime;
  IrmpVersion : THidIrmpVersion;
  MinRepeats : THidReportMinRepeats;

implementation

{$R *.dfm}
function GetIRProtocol(Index: Integer): String;
var
  i : Integer;
begin
  Result := 'Undefined';
  for i := 0 to High(IRMP_PROTOCOLS) do
    if IRMP_PROTOCOLS[i].Protocol = Index then
    begin
      Result := IRMP_PROTOCOLS[i].Name;
      Break;
    end;
end;

procedure TMainForm.OnDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
  const Data: Pointer; Size: Word);
begin
  if ReportID = 1 then
  begin
  CopyMemory(@ReceivedData, Data, Size);
  Memo1.Lines.Add(
    'Protocol : ' + GetIRProtocol(ReceivedData.Protocol)
    + ', Address: 0x' + IntToHex(ReceivedData.Address,4)
    + ', Command: 0x' + IntToHex(ReceivedData.Command,4)
    + ', Flags: 0x' + IntToHex(ReceivedData.Flags,2)
    );
  end;
end;

procedure TMainForm.btPowerOnDisableClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(PowerOnReport, SizeOf(PowerOnReport), 0);
  PowerOnReport.ReportID := SetPowerOnEnabled;
  PowerOnReport.Data := False;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(PowerOnReport, SizeOf(PowerOnReport));
end;

procedure TMainForm.btPowerOnEnableClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(PowerOnReport, SizeOf(PowerOnReport), 0);
  PowerOnReport.ReportID := SetPowerOnEnabled;
  PowerOnReport.Data := True;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(PowerOnReport, SizeOf(PowerOnReport));
end;

procedure TMainForm.btPowerOnEnabledClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(PowerOnReport, SizeOf(PowerOnReport), 0);
  PowerOnReport.ReportID := ReadPowerOnEnabled;

  if Assigned(HIDDevice) then
    if HIDDevice.GetFeature(PowerOnReport, SizeOf(PowerOnReport)) then
      Memo1.Lines.Add(
    'PowerOn Function status: '
    + IntToStr(Integer(PowerOnReport.Data))
    );
end;

procedure TMainForm.btReadIrmpVersionClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(IrmpVersion, SizeOf(IrmpVersion), 0);
  IrmpVersion.ReportID := ReadIrmpVersion;

  if Assigned(HIDDevice) then
    if HIDDevice.GetFeature(IrmpVersion, SizeOf(IrmpVersion)) then
      Memo1.Lines.Add(
    'Irmp Version: '
    + PAnsiChar(StrPas(IrmpVersion.Data))
    );
end;

procedure TMainForm.btReadTrainedIRCodeClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(IRReport, SizeOf(IRReport), 0);
  IRReport.ReportID := ReadTrainendIRCode;

  if Assigned(HIDDevice) then
    if HIDDevice.GetFeature(IRReport, SizeOf(IRReport)) then
      Memo1.Lines.Add(
    'Trained IR Code: '
    + 'Protocol : ' + GetIRProtocol(IRReport.Data.Protocol)
    + ', Address: 0x' + IntToHex(IRReport.Data.Address,4)
    + ', Command: 0x' + IntToHex(IRReport.Data.Command,4)
    + ', Flags: 0x' + IntToHex(IRReport.Data.Flags,2)
    );
end;

procedure TMainForm.btSendIRCodeClick(Sender: TObject);
var
  i : Integer;
begin
  //clear Reports:
  FillChar(IRReport, SizeOf(IRReport), 0);
  IRReport.ReportID := SndIRCodeOut;

  frmSendIRCode.cbIRProtocol.Clear;
  for i := 0 to High(IRMP_PROTOCOLS) do
    frmSendIRCode.cbIRProtocol.Items.Add(IRMP_PROTOCOLS[i].Name);

  frmSendIRCode.ShowModal;

  if IRReport.Data.Protocol > 0 then
  begin
    if Assigned(HIDDevice) then
      HIDDevice.SetFeature(IRReport, SizeOf(IRReport));
  end;
end;

procedure TMainForm.btSetIRCodeClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(IRReport, SizeOf(IRReport), 0);
  IRReport.ReportID := SetTrainedIRCode;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(IRReport, SizeOf(IRReport));
end;

procedure TMainForm.OnChange_IR_Pooling(Sender: TObject);
begin
  Label2.Caption := IntToStr(TrackBar_IR_Polling.Position) + 's';
end;

procedure TMainForm.OnChanged_MinRepeats(Sender: TObject);
begin
  //clear Reports:
  FillChar(MinRepeats, SizeOf(MinRepeats), 0);
  MinRepeats.ReportID := SetMinRepeats;
  MinRepeats.Data := TrackBar_MinRepeats.Position;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(MinRepeats, SizeOf(MinRepeats));
end;

procedure TMainForm.OnChange_MinRepeats(Sender: TObject);
begin
  Label3.Caption := IntToStr(TrackBar_MinRepeats.Position);
end;

procedure TMainForm.OnChanged_IR_Pooling(Sender: TObject);
var
  newFrequency : Extended;
  OCR1A : Word;
begin
  newFrequency := 1 / (TrackBar_IR_Polling.Position / 1000000);
  OCR1A := Trunc(newFrequency);

  //clear Reports:
  FillChar(ReportIRPollingTime, SizeOf(ReportIRPollingTime), 0);
  ReportIRPollingTime.ReportID := SetIRPollingTime;
  ReportIRPollingTime.Data := OCR1A;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(ReportIRPollingTime, SizeOf(ReportIRPollingTime));
end;

procedure TMainForm.OnArrival(HidDev: TJvHidDevice);
var
  i : integer;
begin
  if ((HidDev.Attributes.VendorID = VendorID) AND
    (HidDev.Attributes.ProductID = ProductID) AND
    (HidDev.ProductName = ProductName )) then
  begin

    //store device:
    Memo1.Lines.Add('USB IR Remote Receiver connected');
    HIDDevice := HidDev;
    //device found, lets do the init:
    if HIDDevice.CheckOut then
      begin
        //set feature type
        HIDDevice.ReportTypeParam := HidP_Feature;
        HIDDevice.UsagePageParam := HID_USAGE_PAGE_GENERIC;
        HIDDevice.LinkCollectionParam := $01; // CollectionType=Application
        HIDDevice.UsageParam := HID_USAGE_CONSUMER_UNDEFINED;
      end;
  end;
end;

procedure TMainForm.OnRemoval(HidDev: TJvHidDevice);
begin
  if ((Assigned(HIDDevice)) and (NOT HIDDevice.IsPluggedIn)) then
  begin
    JvHidDeviceController.CheckIn(HIDDevice);
    Memo1.Lines.Add('USB IR Remote Receiver disconnected');
  end;
end;

end.
